from collective.collectionfilter.baseviews import BaseSortOnView
from collective.collectionfilter.interfaces import (  # noqa
    ICollectionFilterResultListSort,
)
from collective.collectionfilter.tiles import BaseFilterTile
from plone.supermodel.model import Schema
from zope.interface import implementer


class ISortOnTile(Schema, ICollectionFilterResultListSort):
    pass


@implementer(ISortOnTile)
class SortOnTile(BaseFilterTile, BaseSortOnView):
    """Sorting Tile"""



"""Setup tests for this package."""
from collective.collectionfilter.testing import (  # noqa
    COLLECTIVE_COLLECTIONFILTER_INTEGRATION_TESTING,
)

import unittest


class TestMaps(unittest.TestCase):
    layer = COLLECTIVE_COLLECTIONFILTER_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        self.collection = self.portal["testcollection"]

    def test_locationfilter(self):
        self.assertEqual(len(self.portal["testcollection"].results()), 6)



from collective.collectionfilter.baseviews import BaseFilterView
from collective.collectionfilter.interfaces import ICollectionFilterSchema
from collective.collectionfilter.tiles import BaseFilterTile
from plone.supermodel.model import Schema
from zope.interface import implementer


class IFilterTile(Schema, ICollectionFilterSchema):
    pass


@implementer(IFilterTile)
class FilterTile(BaseFilterTile, BaseFilterView):
    """Filter Tile"""



from collective.collectionfilter.testing import (  # noqa
    COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_AJAX_DISABLED,
)
from collective.collectionfilter.testing import (
    COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_AJAX_ENABLED,
)
from collective.collectionfilter.testing import (
    COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_TILES,
)
from plone import api
from plone.app.testing import ROBOT_TEST_LEVEL
from plone.testing import layered

import os
import robotsuite
import unittest


def test_suite():
    suite = unittest.TestSuite()
    current_dir = os.path.abspath(os.path.dirname(__file__))
    robot_dir = os.path.join(current_dir, "robot")
    robot_tests = [
        os.path.join("robot", doc)
        for doc in os.listdir(robot_dir)
        if doc.endswith(".robot") and doc.startswith("test_")
    ]
    for robot_test in robot_tests:
        if api.env.plone_version() < "5.1" and "ajaxenabled" in robot_test:
            continue
        elif "ajaxenabled" in robot_test:
            test_layer = (
                (
                    ROBOT_TEST_LEVEL,
                    COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_AJAX_ENABLED,
                ),
            )
        elif "ajaxdisabled" in robot_test:
            test_layer = (
                (
                    ROBOT_TEST_LEVEL,
                    COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_AJAX_DISABLED,
                ),
            )
        elif "_tiles" in robot_test:
            test_layer = (
                (
                    ROBOT_TEST_LEVEL,
                    COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_TILES,
                ),
            )
        elif api.env.plone_version() < "5.1":
            test_layer = (
                (
                    ROBOT_TEST_LEVEL,
                    COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_TILES,
                ),
                (
                    ROBOT_TEST_LEVEL,
                    COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_AJAX_DISABLED,
                ),
            )
        else:
            # We will run generic tests with and without ajax to test everything
            test_layer = (
                (
                    ROBOT_TEST_LEVEL,
                    COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_TILES,
                ),
                (
                    ROBOT_TEST_LEVEL,
                    COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_AJAX_DISABLED,
                ),
                (
                    ROBOT_TEST_LEVEL,
                    COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_AJAX_ENABLED,
                ),
            )
        for level, layer in test_layer:
            robottestsuite = robotsuite.RobotTestSuite(robot_test)
            robottestsuite.level = level
            suite.addTests([layered(robottestsuite, layer=layer)])

    return suite



from collective.collectionfilter.query import make_query


def set_content_filter(context, event):
    """Set the content filter dictionary on the request, built from request
    parameters to narrow the results of the collection.
    """
    req = event.request
    if "collectionfilter" not in req.form or "contentFilter" in req:
        return
    # We leave collectionfilter=1 in req.form so that it gets put into batch links
    content_filter = make_query(req.form)
    event.request["contentFilter"] = content_filter



from collective.collectionfilter import _
from collective.collectionfilter.interfaces import ICollectionish
from collective.collectionfilter.interfaces import IGroupByCriteria
from collective.collectionfilter.query import make_query
from collective.collectionfilter.utils import base_query
from collective.collectionfilter.utils import safe_decode
from collective.collectionfilter.utils import safe_encode
from collective.collectionfilter.utils import safe_iterable
from collective.collectionfilter.vocabularies import DEFAULT_FILTER_TYPE
from collective.collectionfilter.vocabularies import EMPTY_MARKER
from Missing import Missing
from plone.app.contenttypes.behaviors.collection import ICollection
from plone.app.event.base import _prepare_range
from plone.app.event.base import guess_date_from
from plone.app.event.base import start_end_from_mode
from plone.app.event.base import start_end_query
from plone.app.uuid.utils import uuidToObject
from plone.i18n.normalizer import idnormalizer
from plone.memoize import ram
from plone.memoize.volatile import DontCache
from urllib.parse import urlencode
from zope.component import getUtility
from zope.globalrequest import getRequest
from zope.i18n import translate
from zope.interface import implementer

import plone.api


try:
    from plone.app.event.browser.event_listing import EventListing
except ImportError:

    class EventListing:
        pass


def _build_url(
    collection_url, urlquery, filter_value, current_idx_value, idx, filter_type
):
    # Build filter url query
    _urlquery = urlquery.copy()

    # Allow deselection
    if filter_value in current_idx_value:
        _urlquery[idx] = [it for it in current_idx_value if it != filter_value]
    elif filter_type != "single":
        # additive filter behavior
        _urlquery[idx] = current_idx_value + [filter_value]
        _urlquery[idx + "_op"] = filter_type  # additive operator
    else:
        _urlquery[idx] = filter_value

    query_param = urlencode(safe_encode(_urlquery), doseq=True)
    return "/".join(
        [
            it
            for it in [collection_url, "?" + query_param if query_param else None]
            if it
        ]
    )


def _build_option(filter_value, url, current_idx_value, groupby_options):
    idx = groupby_options["index"]
    # Optional modifier to set title from filter value
    display_modifier = groupby_options.get("display_modifier", None)
    # CSS modifier to set class on filter item
    css_modifier = groupby_options.get("css_modifier", None)

    # Set title from filter value with modifications,
    # e.g. uuid to title
    title = filter_value
    if filter_value is not EMPTY_MARKER and callable(display_modifier):
        title = display_modifier(filter_value, idx)
        title = safe_decode(title)

    # Set selected state
    selected = filter_value in current_idx_value
    css_class = "filterItem {}{} {}".format(
        "filter-" + idnormalizer.normalize(filter_value),
        " selected" if selected else "",
        css_modifier(filter_value) if css_modifier else "",
    )

    return {
        "title": title,
        "url": url,
        "value": filter_value,
        "css_class": css_class,
        "count": 1,
        "selected": selected,
    }


def _results_cachekey(
    method,
    target_collection,
    group_by,
    filter_type=DEFAULT_FILTER_TYPE,
    narrow_down=False,
    show_count=False,
    view_name="",
    cache_enabled=True,
    request_params=None,
    content_selector="",
    reverse=False,
):
    if not cache_enabled:
        raise DontCache
    cachekey = (
        target_collection,
        group_by,
        filter_type,
        narrow_down,
        show_count,
        view_name,
        request_params,
        content_selector,
        reverse,
        " ".join(plone.api.user.get_roles()),
        plone.api.portal.get_current_language(),
        str(plone.api.portal.get_tool("portal_catalog").getCounter()),
    )
    return cachekey


@ram.cache(_results_cachekey)
def get_filter_items(
    target_collection,
    group_by,
    filter_type=DEFAULT_FILTER_TYPE,
    narrow_down=False,
    show_count=False,
    view_name="",
    cache_enabled=True,
    request_params=None,
    content_selector="",
    reverse=False,
):
    request_params = request_params or {}
    custom_query = {}  # Additional query to filter the collection

    collection = uuidToObject(target_collection)
    if not collection or not group_by:
        return None
    collection_url = collection.absolute_url()
    option_url = "/".join([it for it in [collection_url, view_name] if it])
    collection = ICollectionish(collection).selectContent(content_selector)
    if (
        collection is None or not collection.content_selector
    ):  # e.g. when no listing tile
        return None

    # Recursively transform all to unicode
    request_params = safe_decode(request_params)

    # Get index in question and the current filter value of this index, if set.
    groupby_criteria = getUtility(IGroupByCriteria).groupby
    idx = groupby_criteria[group_by]["index"]
    current_idx_value = safe_iterable(request_params.get(idx))

    # Additive filtering is about adding other filter values of the same index.
    extra_ignores = [] if narrow_down else [idx, idx + "_op"]

    urlquery = base_query(request_params, extra_ignores)

    # Get all collection results with additional filter defined by urlquery
    custom_query.update(urlquery)
    custom_query = make_query(custom_query)

    catalog_results = collection.results(custom_query, request_params)
    if narrow_down and show_count:
        # we need the extra_ignores to get a true count
        # even when narrow_down filters the display of indexed values
        # count_query allows us to do that true count
        count_query = {}
        count_urlquery = base_query(request_params, [idx, idx + "_op"])
        count_query.update(count_urlquery)
        catalog_results_fullcount = collection.results(count_query, request_params)
    if not catalog_results:
        return None

    # Attribute name for getting filter value from brain
    metadata_attr = groupby_criteria[group_by]["metadata"]
    # Value blacklist
    value_blacklist = groupby_criteria[group_by].get("value_blacklist", None) or []
    # Allow value_blacklist to be callables for runtime-evaluation
    value_blacklist = (
        value_blacklist() if callable(value_blacklist) else value_blacklist
    )
    # fallback to title sorted values
    sort_key_function = groupby_criteria[group_by].get(
        "sort_key_function", lambda it: it["title"].lower()
    )

    grouped_results = {}
    for brain in catalog_results:
        # Get filter value
        val = getattr(brain, metadata_attr, None)
        val = val() if callable(val) else val
        # decode it to unicode
        val = safe_decode(val)
        # Make sure it's iterable, as it's the case for e.g. the subject index.
        val = safe_iterable(val)

        for filter_value in val:
            if (
                filter_value is None
                or isinstance(filter_value, Missing)
                or filter_value in value_blacklist
            ):
                continue
            if type(filter_value) == int:
                # if indexed value is an integer, convert to string
                filter_value = str(filter_value)
            if filter_value in grouped_results:
                # Add counter, if filter value is already present
                grouped_results[filter_value]["count"] += 1
                continue

            url = _build_url(
                collection_url=option_url,
                urlquery=urlquery,
                filter_value=filter_value,
                current_idx_value=current_idx_value,
                idx=idx,
                filter_type=filter_type,
            )
            grouped_results[filter_value] = _build_option(
                filter_value=filter_value,
                url=url,
                current_idx_value=current_idx_value,
                groupby_options=groupby_criteria[group_by],
            )

    # Entry to clear all filters
    urlquery_all = {
        k: v for k, v in list(urlquery.items()) if k not in (idx, idx + "_op")
    }
    if narrow_down and show_count:
        # TODO: catalog_results_fullcount is possibly undefined
        catalog_results = catalog_results_fullcount
    ret = [
        {
            "title": translate(
                _("subject_all", default="All"),
                context=getRequest(),
                target_language=plone.api.portal.get_current_language(),
            ),
            "url": "{}/?{}".format(
                collection_url, urlencode(safe_encode(urlquery_all), doseq=True)
            ),
            "value": "all",
            "css_class": "filterItem filter-all",
            "count": len(catalog_results),
            "selected": idx not in request_params,
        }
    ]

    grouped_results = list(grouped_results.values())

    if callable(sort_key_function):
        grouped_results = sorted(grouped_results, key=sort_key_function)

    if reverse:
        grouped_results = reversed(grouped_results)

    ret += grouped_results

    return ret


@implementer(ICollectionish)
class CollectionishCollection:
    def __init__(self, context):
        self.context = context
        self.collection = ICollection(self.context)

    def selectContent(self, selector=""):
        """Collections can only have a single content"""
        return self

    @property
    def query(self):
        return self.collection.query

    @property
    def sort_on(self):
        return self.collection.sort_on

    @property
    def sort_order(self):
        return self.collection.sort_order

    @property
    def sort_reversed(self):
        return self.collection.sort_reversed

    @property
    def limit(self):
        return self.collection.limit

    @property
    def item_count(self):
        return self.collection.item_count

    @property
    def content_selector(self):
        return "#content-core"  # TODO: could look it up based on view?

    def results(self, custom_query, request_params):
        # Support for the Event Listing view from plone.app.event
        collection_layout = self.context.getLayout()
        default_view = self.context.restrictedTraverse(collection_layout)
        if isinstance(default_view, EventListing):
            mode = request_params.get("mode", "future")
            date = request_params.get("date", None)
            date = guess_date_from(date) if date else None
            start, end = start_end_from_mode(mode, date, self.collection)
            start, end = _prepare_range(self.collection, start, end)
            custom_query.update(start_end_query(start, end))
            # TODO: expand events. better yet, let collection.results
            #        do that

        return self.collection.results(
            batch=False, brains=True, custom_query=custom_query
        )



from collective.collectionfilter.baseviews import BaseMapsView
from collective.collectionfilter.interfaces import ICollectionMapsSchema
from collective.collectionfilter.tiles import BaseFilterTile
from plone.supermodel.model import Schema
from zope.interface import implementer


class IMapsTile(Schema, ICollectionMapsSchema):
    """schema for maps tile"""


@implementer(IMapsTile)
class MapsTile(BaseFilterTile, BaseMapsView):
    """Maps View"""



from collective.collectionfilter import _
from collective.collectionfilter import utils
from plone.api.portal import get_registry_record as getrec
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.app.z3cform.widget import SelectFieldWidget
from plone.autoform.directives import widget
from zope import schema
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


try:
    from plone.formwidget.geolocation.vocabularies import default_map_layer
    from plone.formwidget.geolocation.vocabularies import default_map_layers

    HAS_GEOLOCATION = True
except ImportError:
    HAS_GEOLOCATION = False


def pattern_options():
    options = {
        "basePath": utils.target_collection_base_path,
        "recentlyUsed": True,
        "selectableTypes": utils.target_collection_types,
    }
    return options


class ICollectionish(Interface):
    "Adapts object similar to ICollection if has contentlisting tile, or is a collection"


class ICollectionFilterBaseSchema(Interface):
    header = schema.TextLine(
        title=_("label_header", default="Filter title"),
        description=_("help_header", "Title of the rendered filter."),
        required=False,
    )

    target_collection = schema.Choice(
        title=_("label_target_collection", default="Alternative Target Collection"),
        description=_(
            "help_target_collection",
            default="We use the current context as collection. As an alternative you can select a different "
            "collection as source for the filter items "
            "and where the filter is applied.",
        ),
        required=False,
        vocabulary="plone.app.vocabularies.Catalog",
    )
    widget(
        "target_collection", RelatedItemsFieldWidget, pattern_options=pattern_options()
    )

    view_name = schema.TextLine(
        title=_("label_view_name", default="Result listing view name"),
        description=_(
            "help_view_name",
            default="Optional view name, if the result listing should be"
            " rendered with a special view. Can be used to direct the request"
            " to a tile.",
        ),
        required=False,
        default=None,
    )

    content_selector = schema.TextLine(
        title=_("label_content_selector", default="Content Selector"),
        description=_(
            "help_content_selector",
            default="If your tile or collection has a special class or id for ajax replacement use it here."
            " Selector will need to work for unthemed view and current page.",
        ),
        required=False,
    )


class ICollectionFilterSchema(ICollectionFilterBaseSchema):
    """Schema for the filter."""

    group_by = schema.Choice(
        title=_("label_groupby", "Group by"),
        description=_(
            "help_groupby", "Select the criteria to group the collection results by."
        ),
        required=True,
        vocabulary="collective.collectionfilter.GroupByCriteria",
    )

    show_count = schema.Bool(
        title=_("label_show_count", default="Show count"),
        description=_(
            "help_show_count", default="Show the result count for each filter group."
        ),
        default=False,
        required=False,
    )

    cache_enabled = schema.Bool(
        title=_("label_cache_enabled", default="Enable Cache"),
        description=_(
            "help_cache_enabled",
            default="Enable caching of filter items. The cache is cleared as"
            " soon as the database has any changes.",
        ),
        default=True,
        required=False,
    )

    filter_type = schema.Choice(
        title=_("label_filter_type", "Filter Type"),
        description=_(
            "help_filter_type",
            "Select if single or multiple criteria can be selected and if all (and) or any (or) of the selected criteria must be met."  # noqa
            "Some index types like ``FieldIndex`` (e.g. Type index) only support the any (or) criteria when set to multiple criteria and ignore, if all (and) is set.",  # noqa
        ),
        required=True,
        vocabulary="collective.collectionfilter.FilterType",
    )

    input_type = schema.Choice(
        title=_("label_input_type", "Input Type"),
        description=_(
            "help_input_type",
            "Select how the UI of the collection filter should be rendered. "
            "Whether as links, as checkboxes and radiobuttons or checkboxes and dropdowns.",  # noqa
        ),
        required=True,
        vocabulary="collective.collectionfilter.InputType",
    )

    reverse = schema.Bool(
        title=_("label_reverse", default="Reverse sort filter"),
        description=_(
            "help_reverse",
            default="Reverse the sorting of th list of filter options.",
        ),
        default=False,
        required=False,
    )

    narrow_down = schema.Bool(
        title=_("label_narrow_down", default="Narrow down filter options"),
        description=_(
            "help_narrow_down",
            default="Narrow down the filter options when a filter of this group is applied."  # noqa
            " Only options, which are available in the result set will then be displayed."  # noqa
            " Other filter groups can still narrow down this one, though.",
        ),  # noqa
        default=False,
        required=False,
    )

    hide_if_empty = schema.Bool(
        title=_("label_hide_if_empty", default="Hide if empty"),
        description=_(
            "help_hide_if_empty",
            default="Don't display if there is 1 or no options without selecting a filter yet.",
        ),
        default=False,
        required=False,
    )


#    list_scaling = schema.Choice(
#        title=_('label_list_scaling', u'List scaling'),
#        description=_(
#            'help_list_scaling',
#            u'Scale list by count. If a scaling is selected, a the list '
#            u'appears as tagcloud.'
#        ),
#        required=True,
#        vocabulary='collective.collectionfilter.ListScaling',
#    )


class ICollectionSearchSchema(ICollectionFilterBaseSchema):
    """Schema for the search filter."""


class ICollectionFilterResultListSort(ICollectionFilterBaseSchema):
    """Schema for the result list sorting."""

    sort_on = schema.Tuple(
        title=_("label_sort_on", "Enabled sort indexes"),
        description=_("help_sort_on", "Select the indexes which can be sorted on."),
        value_type=schema.Choice(
            title="Index",
            vocabulary="collective.collectionfilter.SortOnIndexes",
        ),
        required=True,
    )
    # NB needed as InAndOut breaks tiles in 5.0
    widget("sort_on", SelectFieldWidget, pattern_options=dict(orderable=True))

    input_type = schema.Choice(
        title=_("label_input_type", "Input Type"),
        description=_(
            "help_input_type",
            "Select how the UI of the collection filter should be rendered. "
            "Whether as links, as checkboxes and radiobuttons or checkboxes and dropdowns.",  # noqa
        ),
        required=True,
        vocabulary="collective.collectionfilter.InputType",
    )


class IGroupByCriteria(Interface):
    """Interface for the GroupByCriteria utility.

    groupby provides a datastructure like this::

        GROUPBY_CRITERIA = {
            'Subject': {
                'index': 'Subject',      # Index for querying.
                'metadata': 'Subject',   # Metadata name for fast access.
                'display_modifier': _,   # Function for modifying list items
                                         # for display. Gets the item passed
                'css_modifier': None,    # Change css class of filter item
                'index_modifier': None,  # Change index values before querying
                'value_blacklist': None  # Exclude index values from display
                'sort_key_function': None,  # sort key function. defaults to a lower-cased title.  # noqa
            },
        }
    """


class IGroupByModifier(Interface):
    """Adapter interface for modifying the groupby criteria data structure,
    after it has been initially created.
    """


class ICollectionFilterBrowserLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


if HAS_GEOLOCATION:

    def map_layer_default():
        return getrec(name="geolocation.default_map_layer", default=default_map_layer)

    def map_layers_default():
        return getrec(name="geolocation.map_layers", default=default_map_layers)

    class ICollectionMapsSchema(ICollectionFilterBaseSchema):
        """schema for maps filtering"""

        narrow_down = schema.Bool(
            title=_("label_narrow_down_results", default="Narrow down result"),
            description=_(
                "help_narrow_down_results",
                default="Narrow down the result after zooming/moving the map.",
            ),
            default=False,
            required=False,
        )

        default_map_layer = schema.Choice(
            title=_("default_map_layer", "Default map layer"),
            description=_(
                "help_default_map_layer", default="Set the default map layer"
            ),
            required=False,
            defaultFactory=map_layer_default,
            vocabulary="plone.formwidget.geolocation.vocabularies.map_layers",
        )

        map_layers = schema.List(
            title=_("label_map_layers", "Map Layers"),
            description=_("help_map_layers", default="Set the available map layers"),
            required=False,
            defaultFactory=map_layers_default,
            missing_value=[],
            value_type=schema.Choice(
                vocabulary="plone.formwidget.geolocation.vocabularies.map_layers"
            ),
        )

        geojson_properties_limit = schema.Int(
            title=_("geojson_properties_limit", default="Limit for GeoJSON properties"),
            description=_(
                "help_geojson_properties_limit",
                default="If the search result is larger than this limit, no additional "
                "GeoJSON properties (like popup information) are shown.",
            ),
            required=False,
            default=500,
        )



from collective.collectionfilter.portlets.collectionfilter import (  # noqa
    ICollectionFilterPortlet,
)
from plone.portlets.interfaces import ILocalPortletAssignable
from plone.portlets.interfaces import IPortletAssignmentMapping
from plone.portlets.interfaces import IPortletManager
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.GenericSetup.interfaces import ISetupTool
from zope.component import getMultiAdapter
from zope.component import getUtilitiesFor
from zope.component import getUtility
from zope.component.hooks import getSite

import logging


_marker = []

logger = logging.getLogger(__file__)


def upgrade_portlet_input_type(ctx):
    # See: https://docs.plone.org/4/en/old-reference-manuals/portlets/appendix/schema_update.html
    context = getSite()
    cat = getToolByName(context, "portal_catalog")
    query = {"object_provides": ILocalPortletAssignable.__identifier__}
    all_brains = cat(**query)
    all_content = [brain.getObject() for brain in all_brains]
    all_content.append(context)
    for content in all_content:
        for manager_name, manager in getUtilitiesFor(IPortletManager, context=content):
            mapping = getMultiAdapter(
                (content, manager), IPortletAssignmentMapping
            )  # noqa
            for id, assignment in mapping.items():
                if ICollectionFilterPortlet.providedBy(assignment):
                    as_input = getattr(assignment, "as_input")
                    if as_input:
                        logger.info(
                            "Set {} input_type to ``checkboxes_radiobuttons``".format(
                                assignment
                            )
                        )  # noqa
                        setattr(assignment, "input_type", "checkboxes_radiobuttons")
                    else:
                        logger.info(f"Set {assignment} input_type to ``links``")  # noqa
                        setattr(assignment, "input_type", "links")


def loadMigrationProfile(context, profile, steps=_marker):
    if not ISetupTool.providedBy(context):
        context = getToolByName(context, "portal_setup")
    if steps is _marker:
        context.runAllImportStepsFromProfile(profile, purge_old=False)
    else:
        for step in steps:
            context.runImportStepFromProfile(
                profile, step, run_dependencies=False, purge_old=False
            )


def reapply_profile(context):
    loadMigrationProfile(
        context,
        "profile-collective.collectionfilter:default",
    )


def upgrade_to_plone6(context):
    # Delete deprecated keys in the registry
    keys_to_delete = [
        "plone.bundles/collectionfilter-bundle.compile",
        "plone.bundles/collectionfilter-bundle.develop_css",
        "plone.bundles/collectionfilter-bundle.develop_javascript",
        "plone.bundles/collectionfilter-bundle.last_compilation",
        "plone.bundles/collectionfilter-bundle.merge_with",
        "plone.bundles/collectionfilter-bundle.resources",
        "plone.bundles/collectionfilter-bundle.stub_js_modules",
        "plone.resources/collectionfilter-bundle.conf",
        "plone.resources/collectionfilter-bundle.css",
        "plone.resources/collectionfilter-bundle.deps",
        "plone.resources/collectionfilter-bundle.export",
        "plone.resources/collectionfilter-bundle.init",
        "plone.resources/collectionfilter-bundle.js",
        "plone.resources/collectionfilter-bundle.url",
        "plone.resources/collectionfilter.conf",
        "plone.resources/collectionfilter.css",
        "plone.resources/collectionfilter.deps",
        "plone.resources/collectionfilter.export",
        "plone.resources/collectionfilter.init",
        "plone.resources/collectionfilter.js",
        "plone.resources/collectionfilter.url",
    ]
    registry = getUtility(IRegistry)
    for key in keys_to_delete:
        if key in registry.records.keys():
            del registry.records[key]

    reapply_profile(context)



from collective.collectionfilter import _
from collective.collectionfilter.interfaces import IGroupByCriteria
from collective.collectionfilter.interfaces import IGroupByModifier
from collective.collectionfilter.utils import safe_encode
from plone.app.querystring.interfaces import IQuerystringRegistryReader
from plone.app.vocabularies.types import (  # noqa: E501
    ReallyUserFriendlyTypesVocabularyFactory,
)
from plone.registry.interfaces import IRegistry
from zope.component import getAdapters
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.globalrequest import getRequest
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import provider
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

import plone.api


# Use this EMPTY_MARKER for your custom indexer to index empty criterions.
EMPTY_MARKER = "__EMPTY__"
TEXT_IDX = "SearchableText"
INTEGER_IDXS = []
GEOLOC_IDX = [
    "latitude",
    "longitude",
]
GROUPBY_BLACKLIST = [
    "CreationDate",
    "Date",
    "Description",
    "EffectiveDate",
    "ExpirationDate",
    "ModificationDate",
    "Title",
    "UID",
    "cmf_uid",
    "created",
    "effective",
    "end",
    "expires",
    "getIcon",
    "getId",
    "getObjSize",
    "getRemoteUrl",
    "id",
    "last_comment_date",
    "listCreators",
    "meta_type",
    "modified",
    "start",
    "sync_uid",
    "total_comments",
    "TranslationGroup",
] + GEOLOC_IDX  # latitude/longitude is handled as a range filter ... see query.py  # noqa
DEFAULT_FILTER_TYPE = "single"
LIST_SCALING = ["No Scaling", "Linear", "Logarithmic"]
TRUTHY = [
    safe_encode("true"),
    safe_encode("1"),
    safe_encode("t"),
    safe_encode("yes"),
]


def translate_value(value, *args, **kwargs):
    return translate(_(value), context=getRequest())


def translate_messagefactory(value, *args, **kwargs):
    return translate(value, context=getRequest())


def make_bool(value):
    """Transform into a boolean value."""

    if value is None:
        return
    if isinstance(value, bool):
        return value
    value = safe_encode(value).lower()
    return value in TRUTHY


def make_int(value):
    if isinstance(value, (list, tuple)):
        return [int(val) for val in value]
    return int(value)


def yes_no(value):
    """Return i18n message for a value."""
    if value:
        return _("Yes")
    return _("No")


def get_yes_no_title(item, *args, **kwargs):
    """Return a readable representation of a boolean value."""
    value = yes_no(item)
    return translate_messagefactory(value)


def translate_portal_type(value, *args, **kwargs):
    vocabulary = ReallyUserFriendlyTypesVocabularyFactory(None)
    term = vocabulary.getTermByToken(value)
    return term.title if term else value


def sort_key_title(it):
    """default sort key function uses a lower-cased title."""
    title = it["title"]
    if title is None:
        return ""
    return title.lower()


@implementer(IGroupByCriteria)
class GroupByCriteria:
    """Global utility for retrieving and manipulating groupby criteria.

    1) Populate ``groupby`` catalog metadata.
    2) Do not use blacklisted metadata columns.
    3) Use IGroupByModifier adapters to modify the datastructure.

    """

    _groupby = None
    groupby_modify = {}

    @property
    def groupby(self):
        if self._groupby is not None:
            # The groupby criteria are used at each IBeforeTraverseEvent - so
            # on each request. This has to be fast, so exit early.
            return self._groupby
        self._groupby = {}

        cat = plone.api.portal.get_tool("portal_catalog")
        # get catalog metadata schema, but filter out items which cannot be
        # used for grouping
        metadata = [it for it in cat.schema() if it not in GROUPBY_BLACKLIST]
        for it in metadata:
            if it not in cat.indexes():
                # collectionfilter needs both, metadata and index entries.
                continue
            idx = cat._catalog.indexes.get(it)
            index_modifier = None
            display_modifier = translate_value  # Allow to translate in this package domain per default.  # noqa

            if getattr(idx, "meta_type", None) == "BooleanIndex":
                index_modifier = make_bool
                display_modifier = get_yes_no_title

            if idx.getId() in INTEGER_IDXS:
                index_modifier = make_int

            # for portal_type or Type we have some special sauce as we need to translate via fti.i18n_domain.  # noqa
            if it == "portal_type":
                display_modifier = translate_portal_type
            elif it == "Type":
                display_modifier = translate_messagefactory

            self._groupby[it] = {
                "index": it,
                "metadata": it,
                "display_modifier": display_modifier,
                "css_modifier": None,
                "index_modifier": index_modifier,
                "value_blacklist": None,
                "sort_key_function": sort_key_title,
            }

        modifiers = getAdapters((self,), IGroupByModifier)
        for name, modifier in sorted(modifiers, key=lambda it: it[0]):
            modifier()

        return self._groupby

    @groupby.setter
    def groupby(self, value):
        self._groupby = value


@provider(IVocabularyFactory)
def GroupByCriteriaVocabulary(context):
    """Collection filter group by criteria."""
    groupby = getUtility(IGroupByCriteria).groupby
    items = [SimpleTerm(value=it, token=str(it), title=_(it)) for it in groupby.keys()]
    return SimpleVocabulary(items)


# TODO: this should depend on the index type, or be validated against it
@provider(IVocabularyFactory)
def FilterTypeVocabulary(context):
    items = [
        SimpleTerm(title=_("filtertype_single"), value="single"),
        SimpleTerm(title=_("filtertype_and"), value="and"),
        SimpleTerm(title=_("filtertype_or"), value="or"),
    ]
    return SimpleVocabulary(items)


@provider(IVocabularyFactory)
def InputTypeVocabulary(context):
    items = [
        SimpleTerm(title=_("inputtype_links"), value="links"),
        SimpleTerm(
            title=_("inputtype_checkboxes_radiobuttons"),
            value="checkboxes_radiobuttons",
        ),
        SimpleTerm(
            title=_("inputtype_checkboxes_dropdowns"), value="checkboxes_dropdowns"
        ),
    ]
    return SimpleVocabulary(items)


@provider(IVocabularyFactory)
def ListScalingVocabulary(context):
    items = [SimpleTerm(title=_(it), value=it) for it in LIST_SCALING]
    return SimpleVocabulary(items)


@provider(IVocabularyFactory)
def SortOnIndexesVocabulary(context):
    # we reuse p.a.querystring registry reader for sortable_indexes
    registry = getUtility(IRegistry)
    reader = getMultiAdapter((registry, getRequest()), IQuerystringRegistryReader)
    sortable_indexes = reader().get("sortable_indexes")
    items = [
        SimpleTerm(title=_(v["title"]), value=k) for k, v in sortable_indexes.items()
    ]
    return SimpleVocabulary(items)



from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles:
    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation."""
        return [
            "collective.collectionfilter:uninstall",
            "collective.collectionfilter.tiles:uninstall",
            "collective.collectionfilter.portlets:uninstall",
        ]



from plone.app.portlets.portlets.base import Renderer
from plone.base.utils import get_top_request
from plone.base.utils import safe_text


class BasePortletRenderer(Renderer):
    @property
    def filter_id(self):
        request = get_top_request(self.request)
        portlethash = request.form.get(
            "portlethash", getattr(self, "__portlet_metadata__", {}).get("hash", "")
        )
        return portlethash

    @property
    def reload_url(self):
        reload_url = "{}/@@render-portlet?portlethash={}".format(
            self.context.absolute_url(),
            safe_text(self.filter_id),
        )
        return reload_url

    @property
    def available(self):
        return self.is_available



TEST_COLLECTION_ID = "collection-filter-test"
PLONE_PATH = "/plone"
COLLECTION_TEST_QUERY = (
    '[{"i": "path", "o": "plone.app.querystring.operation.string.path", "v": "%s"}]'
    % (PLONE_PATH)
)  # noqa



from collective.collectionfilter.interfaces import IGroupByCriteria
from collective.collectionfilter.utils import safe_decode
from collective.collectionfilter.vocabularies import EMPTY_MARKER
from collective.collectionfilter.vocabularies import GEOLOC_IDX
from collective.collectionfilter.vocabularies import TEXT_IDX
from logging import getLogger
from plone import api
from Products.CMFPlone.browser.search import BAD_CHARS
from Products.CMFPlone.browser.search import quote_chars
from Products.CMFPlone.UnicodeSplitter.config import rxGlob_U
from zope.component import getUtility


try:
    from Products.CMFPlone.browser.search import quote
except ImportError:
    # This fix was not in Plone 5.0.x
    def quote(term):
        # The terms and, or and not must be wrapped in quotes to avoid
        # being parsed as logical query atoms.
        if term.lower() in ("and", "or", "not"):
            term = '"%s"' % term
        return term


logger = getLogger("collective.collectionfilter")
ENCODED_BAD_CHARS = safe_decode(BAD_CHARS)


def sanitise_search_query(query):
    if not rxGlob_U.findall(query):
        return ""
    for char in ENCODED_BAD_CHARS:
        query = query.replace(char, " ")
    clean_query = [quote(token) for token in query.split()]
    clean_query = quote_chars(clean_query)
    return " ".join(clean_query)


def make_query(params_dict):
    """Make a query from a dictionary of parameters, like a request form."""
    query_dict = {}
    groupby_criteria = getUtility(IGroupByCriteria).groupby
    cat = api.portal.get_tool("portal_catalog")

    for val in groupby_criteria.values():
        idx = val["index"]
        if idx in params_dict:
            crit = params_dict.get(idx) or EMPTY_MARKER
            cat_idx = cat._catalog.indexes.get(idx)

            idx_mod = val.get("index_modifier", None)
            crit = idx_mod(crit) if idx_mod else safe_decode(crit)

            # filter operator
            op = params_dict.get(idx + "_op", None)
            idx_has_operator = "operator" in getattr(
                cat_idx,
                "query_options",
                [
                    "query",
                ],
            )  # noqa: E501

            if op is None or not idx_has_operator:
                # add filter query
                query_dict[idx] = {"query": crit}
            else:
                if op not in ["and", "or"]:
                    op = "or"
                # add filter query
                query_dict[idx] = {"operator": op, "query": crit}

    for idx in GEOLOC_IDX:
        if idx in params_dict:
            # lat/lng query has to be float values
            try:
                query_dict[idx] = dict(
                    query=[
                        float(params_dict[idx]["query"][0]),
                        float(params_dict[idx]["query"][1]),
                    ],
                    range=params_dict[idx]["range"],
                )
            except (ValueError, TypeError):
                logger.warning(
                    "Could not apply lat/lng values to filter: %s", params_dict[idx]
                )

    if TEXT_IDX in params_dict and params_dict.get(TEXT_IDX):
        safe_text = safe_decode(params_dict.get(TEXT_IDX))
        clean_searchable_text = sanitise_search_query(safe_text)
        query_dict[TEXT_IDX] = clean_searchable_text

    if "sort_on" in params_dict:
        query_dict["sort_on"] = params_dict["sort_on"]
        query_dict["sort_order"] = params_dict.get("sort_order", "ascending")

    return query_dict



from Acquisition import aq_chain
from Acquisition import aq_inner
from collective.collectionfilter import _
from collective.collectionfilter.filteritems import ICollectionish
from collective.collectionfilter.interfaces import ICollectionFilterBaseSchema
from collective.collectionfilter.interfaces import ICollectionFilterBrowserLayer
from plone.app.uuid.utils import uuidToCatalogBrain
from plone.dexterity.interfaces import IDexterityContent
from plone.portlets.interfaces import IPortletAssignmentMapping
from z3c.form import validator
from z3c.form.interfaces import IValidator
from zope.component import adapter
from zope.component import queryAdapter
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import Invalid
from zope.schema.interfaces import IField


# Portlets can always validate.
# Tiles can validate if its a collection, otherwise rely on status message warning on save.


@implementer(IValidator)
@adapter(Interface, ICollectionFilterBrowserLayer, Interface, IField, Interface)
class TargetCollectionValidator(validator.SimpleFieldValidator):
    def validate(self, value):
        super().validate(value)
        portlet = False
        if value:
            obj = aq_inner(uuidToCatalogBrain(value)).getObject()
        else:
            for obj in aq_chain(self.context):
                if IPortletAssignmentMapping.providedBy(obj):
                    portlet = True
                if IDexterityContent.providedBy(obj):
                    break
        collection = queryAdapter(obj, ICollectionish)
        # if it's a tile we will use a warning instead since we can't tell if a listing tile
        # has been added to the layout yet as it's not saved.
        if portlet and (collection is None or collection.content_selector is None):
            raise Invalid(
                _(
                    "Context is not a collection or has a contentlisting tile, please set a target."
                )
            )
        return True


validator.WidgetValidatorDiscriminators(
    TargetCollectionValidator,
    field=ICollectionFilterBaseSchema["target_collection"],
)
